package com.tzld.longarticle.recommend.server.config.db;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.boot.orm.jpa.EntityManagerFactoryBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;

@Configuration
@EnableTransactionManagement
@EnableJpaRepositories(
        basePackages = "com.tzld.longarticle.recommend.server.repository.crawler",
        entityManagerFactoryRef = "crawlerEntityManagerFactory",
        transactionManagerRef = "crawlerTransactionManager"
)
public class CrawlerDBConfig {

    @Value("${spring.jpa.crawler.hibernate.ddl-auto}")
    private String ddlAuto;
    @Value("${spring.jpa.crawler.database}")
    private String database;

    public Map<String, Object> hibernateProperties() {
        Map<String, Object> properties = new HashMap<>();
        properties.put("hibernate.ddl-auto", ddlAuto);
        properties.put("database", database);
        return properties;
    }

    @Primary
    @Bean(name = "crawlerDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.crawler")
    public DataSource crawlerDataSource() {
        return DataSourceBuilder.create().build();
    }

    @Primary
    @Bean(name = "crawlerEntityManagerFactory")
    public LocalContainerEntityManagerFactoryBean crawlerEntityManagerFactory(EntityManagerFactoryBuilder builder,
                                                                              @Qualifier("crawlerDataSource") DataSource dataSource) {
        return builder
                .dataSource(dataSource)
                .packages("com.tzld.longarticle.recommend.server.repository.entity.crawler") // 实体类包路径
                .persistenceUnit("crawler")
                .properties(hibernateProperties())
                .build();
    }

    @Primary
    @Bean(name = "crawlerTransactionManager")
    public PlatformTransactionManager crawlerTransactionManager(
            @Qualifier("crawlerEntityManagerFactory") EntityManagerFactory crawlerEntityManagerFactory) {
        return new JpaTransactionManager(crawlerEntityManagerFactory);
    }
}

