package com.tzld.longarticle.recommend.server.remote;

import com.alibaba.fastjson.JSONObject;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.tms.v20201229.TmsClient;
import com.tencentcloudapi.tms.v20201229.models.TextModerationRequest;
import com.tencentcloudapi.tms.v20201229.models.TextModerationResponse;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleSensitiveRepository;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.ArticleSensitive;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Base64;
import java.util.Map;
import java.util.Objects;

@Service
@Slf4j
public class ArticleSensitiveRemoteService {

    @Autowired
    ArticleSensitiveRepository articleSensitiveRepository;

    @Value("${tecent.cloud.secret.id:AKIDbLGZox42mZaloeIo16zeJrDRUelkbFfw}")
    private String tecentCloudSecretId;
    @Value("${tecent.cloud.secret.key:CFArLAnCONHLany1qRrLyW2Rmw3ITXi1}")
    private String tecentCloudSecretKey;

    public Boolean articleSensitive(String title, Map<String, String> titleMd5Map, Map<String, ArticleSensitive> articleSensitiveMap) {
        long start = System.currentTimeMillis();
        boolean isSensitive = false;
        String md5 = titleMd5Map.get(title);
        ArticleSensitive articleSensitive = articleSensitiveMap.get(md5);
        if (Objects.nonNull(articleSensitive) && StringUtils.hasText(articleSensitive.getSensitiveSuggestion())) {
            if ("Pass".equals(articleSensitive.getSensitiveSuggestion())
                    || ("Polity".equals(articleSensitive.getLabel()) && "ForeignLeader".equals(articleSensitive.getSubLabel()))) {
                return false;
            } else {
                return true;
            }
        }
        String content = Base64.getEncoder().encodeToString(title.getBytes());
        try {
            // 实例化一个认证对象，入参需要传入腾讯云账户 SecretId 和 SecretKey，此处还需注意密钥对的保密
            // 代码泄露可能会导致 SecretId 和 SecretKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考，建议采用更安全的方式来使用密钥，请参见：https://cloud.tencent.com/document/product/1278/85305
            // 密钥可前往官网控制台 https://console.cloud.tencent.com/cam/capi 进行获取
            Credential cred = new Credential(tecentCloudSecretId, tecentCloudSecretKey);
            // 实例化一个http选项，可选的，没有特殊需求可以跳过
            HttpProfile httpProfile = new HttpProfile();
            httpProfile.setEndpoint("tms.tencentcloudapi.com");
            // 实例化一个client选项，可选的，没有特殊需求可以跳过
            ClientProfile clientProfile = new ClientProfile();
            clientProfile.setHttpProfile(httpProfile);
            // 实例化要请求产品的client对象,clientProfile是可选的
            TmsClient client = new TmsClient(cred, "ap-beijing", clientProfile);
            // 实例化一个请求对象,每个接口都会对应一个request对象
            TextModerationRequest req = new TextModerationRequest();
            req.setContent(content);
            // 返回的resp是一个TextModerationResponse的实例，与请求对象对应
            TextModerationResponse resp = client.TextModeration(req);
            log.info("articleSensitive title:{} response:{}", title, JSONObject.toJSONString(resp));
            if ("Pass".equals(resp.getSuggestion())
                    || ("Polity".equals(resp.getLabel()) && "ForeignLeader".equals(resp.getSubLabel()))) {
            } else {
                isSensitive = true;
            }
            insertArticleSensitive(md5, title, resp.getSuggestion(), resp.getLabel(), resp.getSubLabel(), JSONObject.toJSONString(resp));
        } catch (Exception e) {
            log.error("articleSensitive error", e);
        }
        log.info("articleSensitive耗时：{}", System.currentTimeMillis() - start);
        return isSensitive;
    }

    public ArticleSensitive getArticleSensitive(String md5) {
        return articleSensitiveRepository.findById(md5).orElse(null);
    }

    public void insertArticleSensitive(String md5, String title, String sensitiveSuggestion, String label, String subLabel, String response) {
        // 插入
        ArticleSensitive insert = new ArticleSensitive(md5, title, sensitiveSuggestion, label, subLabel, response);
        articleSensitiveRepository.save(insert);
    }


    public static void main(String[] args) {
        // articleSensitive("特朗普说的双发F-35，听着耳熟，东北上空的试飞，看出歼35新定位");
    }
}

