package com.tzld.longarticle.recommend.server.service;

import com.ctrip.framework.apollo.model.ConfigChangeEvent;
import com.ctrip.framework.apollo.spring.annotation.ApolloConfigChangeListener;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Map;

/**
 * @author dyp
 */
@Component
@Slf4j
public class AccountContentPoolConfigService {

    @ApolloJsonValue("${accountContentPoolConfig:{}}")
    private Map<String, String[]> accountContentPoolMap;

    private BiMap<String, Integer> contentPoolLevelMap;

    @ApolloJsonValue("${contentPoolLevelConfig:{}}")
    private Map<String, Integer> contentPoolLevelConfig;

    @PostConstruct
    public void init() {
        contentPoolLevelMap = HashBiMap.create(contentPoolLevelConfig);
    }


    @ApolloConfigChangeListener(interestedKeys = {"contentPoolLevelConfig"})
    public void configChange(ConfigChangeEvent changeEvent) {
        init();
    }

    public String[] getContentPools(String accountName) {
        if (accountContentPoolMap.containsKey(accountName)) {
            return accountContentPoolMap.get(accountName);
        }
        return accountContentPoolMap.get("default");
    }

    public String getContentPoolByLevel(int level) {

        return contentPoolLevelMap.inverse().getOrDefault(level, "");
    }

    public Integer getLevelByContentPool(String contentPool) {
        return contentPoolLevelMap.get(contentPool);
    }
}
