package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.model.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


@Component
@Slf4j
public class FlowCtlDecreaseStrategy implements ScoreStrategy {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private PublishSortLogRepository publishSortLogRepository;

    // 账号score减少
    @ApolloJsonValue("${accountColdStartScoreDecreaseConfig:{\"default\":{\"3\":-100,\"1\":-100,\"2\":-100}}}")
    private Map<String, Map<String, Integer>> accountColdStartScoreDecreaseMap;

    // 计算阅读均值base
    @ApolloJsonValue("${totalAvgReadCountBase:{\"default\":{\"3\":100,\"1\":100,\"2\":100}}}")
    private Map<String, Map<String, Integer>> totalAvgReadCountBaseMap;

    @Override
    public List<Score> score(ScoreParam param) {
        List<Score> scores = new ArrayList<>();
        if (CollectionUtils.isEmpty(param.getContents())) {
            return scores;
        }
        String dateStr = DateUtils.getCurrentDateStr("yyyy-MM-dd");
        List<String> titles = param.getContents().stream().map(Content::getTitle).collect(Collectors.toList());
        // 获取今日已发布内容
        List<PublishSortLog> hisPublishContentList = publishSortLogRepository.findByDateStrAndTitleIn(dateStr, titles);
        Map<String, List<PublishSortLog>> hisPublishedContentMap = hisPublishContentList.stream().collect(Collectors.groupingBy(PublishSortLog::getTitle));
        for (Content content : param.getContents()) {
            Score score = new Score();
            score.setStrategy(this);
            score.setContentId(content.getId());
            Integer scoreVal = getContentScore(param.getAccountName(), hisPublishedContentMap, content);
            if (scoreVal != 0) {
                score.setScore(scoreVal);
                scores.add(score);
            }
        }

        return scores;
    }

    private Integer getContentScore(String accountName,
                                    Map<String, List<PublishSortLog>> hisPublishedContentMap,
                                    Content content) {
        String[] contentPools = accountContentPoolConfigService.getContentPools(accountName);
        int index = 3;
        for (int i = 0; i < contentPools.length; i++) {
            if (contentPools[i].equals(content.getContentPoolType())) {
                index = i + 1;
                break;
            }
        }
        Map<String, Integer> indexWeight = getColdStartScoreDecreaseWeight(accountName);
        Map<String, Integer> indexTotalAvgReadCountBase = getAvgReadCountBase(accountName);
        if (Objects.isNull(indexWeight)
                || Objects.isNull(indexTotalAvgReadCountBase)
                || !indexWeight.containsKey(String.valueOf(index))
                || !indexTotalAvgReadCountBase.containsKey(String.valueOf(index))) {
            return 0;
        }
        if (hisPublishedContentMap.containsKey(content.getTitle())) {
            List<PublishSortLog> publishContents = hisPublishedContentMap.get(content.getTitle());
            double sumViewCount = publishContents.stream().mapToDouble(PublishSortLog::getIndexAvgCount).sum();
            int hisViewCountSum = content.getHisPublishArticleList().stream().filter(ContentHisPublishArticle::isInnerAccount)
                    .mapToInt(ContentHisPublishArticle::getAvgViewCount).sum();
            if ((sumViewCount + hisViewCountSum) > indexTotalAvgReadCountBase.get(String.valueOf(index))) {
                return indexWeight.get(String.valueOf(index));
            }
        }
        return 0;
    }

    public Map<String, Integer> getColdStartScoreDecreaseWeight(String accountName) {
        if (accountColdStartScoreDecreaseMap.containsKey(accountName)) {
            return accountColdStartScoreDecreaseMap.get(accountName);
        }
        return accountColdStartScoreDecreaseMap.get("default");
    }

    public Map<String, Integer> getAvgReadCountBase(String accountName) {
        if (totalAvgReadCountBaseMap.containsKey(accountName)) {
            return totalAvgReadCountBaseMap.get(accountName);
        }
        return totalAvgReadCountBaseMap.get("default");
    }

}
