package com.tzld.longarticle.recommend.server.util;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class TitleSimilarCheckUtil {

    private static final double SIMILARITY_THRESHOLD = 0.8; // 相似度阈值

    public static boolean isDuplicateContent(String title, List<String> existsContentTitle) {
        boolean result = false;
        title = title.trim().replace("\u200b", "");
        for (String existsTitle : existsContentTitle) {
            if (isSimilar(title, existsTitle, SIMILARITY_THRESHOLD)) {
                return true;
            }
        }
        return result;
    }

    private static boolean isSimilar(String titleA, String titleB, double threshold) {
        if (titleA.isEmpty() || titleB.isEmpty()) {
            return false;
        }
        Set<Character> setA = new HashSet<>();
        for (char c : titleA.toCharArray()) {
            setA.add(c);
        }
        Set<Character> setB = new HashSet<>();
        for (char c : titleB.toCharArray()) {
            setB.add(c);
        }
        Set<Character> setCross = new HashSet<>(setA);
        setCross.retainAll(setB);
        int minLen = Math.max(Math.min(setA.size(), setB.size()), 1);
        double rate = (double) setCross.size() / minLen;
        return rate >= threshold;
    }

    public static void main(String[] args) {
        String title = "多子女家庭，老人大概率过得比独子家庭的要幸福";
        List<String> existsContentTitle = Arrays.asList("以后买房，请记住7字真言：“买旧、买大、不买三！”",
                "人到晚年才明白：多子女家庭，老人大概率过得比独子家庭的要幸福",
                "可供中国使用3800年？山东意外发现巨大宝藏，西方当场酸了！",
                "陕西女孩去医院体检后，发现左肾不见了，意外牵出8年前手术疑云");
        boolean result = isDuplicateContent(title, existsContentTitle);
        System.out.println(result);
    }
}
